/****************************************************************************
 * This file is part of the project AqDataBase.
 * AqFinance (c) by 2015 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifndef AQFS_ITEM_HEADER_H
#define AQFS_ITEM_HEADER_H


#include <stdint.h>



typedef struct AQFS_ITEM_HEADER AQFS_ITEM_HEADER;


#define AQFS_ITEM_HEADER_SIZE                20


enum {
  AQFS_ITEMHEADER_TYPE_DELETED=0,
  AQFS_ITEMHEADER_TYPE_GENERIC_DATA = (1 | ('D'<<8) | ('A'<<16) | ('T'<<24)),
  AQFS_ITEMHEADER_TYPE_GENERIC_TABLE= (2 | ('T'<<8) | ('A'<<16) | ('B'<<24))
};



AQFS_ITEM_HEADER *AQFS_ItemHeader_new();
void AQFS_ItemHeader_free(AQFS_ITEM_HEADER *ih);


int AQFS_ItemHeader_ReadFromBuffer(AQFS_ITEM_HEADER *ih, const uint8_t *buffer, uint64_t len);
AQFS_ITEM_HEADER *AQFS_ItemHeader_fromBuffer(const uint8_t *buffer, uint64_t len);
uint64_t AQFS_ItemHeader_toBuffer(const AQFS_ITEM_HEADER *ih, uint8_t *buffer, uint64_t len);




uint64_t AQFS_ItemHeader_GetSizeAllocated(const AQFS_ITEM_HEADER *ih);
void AQFS_ItemHeader_SetSizeAllocated(AQFS_ITEM_HEADER *ih, uint64_t i);

uint64_t AQFS_ItemHeader_GetSizeUsed(const AQFS_ITEM_HEADER *ih);
void AQFS_ItemHeader_SetSizeUsed(AQFS_ITEM_HEADER *ih, uint64_t i);

uint32_t AQFS_ItemHeader_GetDataType(const AQFS_ITEM_HEADER *ih);
void AQFS_ItemHeader_SetDataType(AQFS_ITEM_HEADER *ih, uint32_t i);


uint64_t AQFS_ItemHeader_GetOffsetHeaderStart(const AQFS_ITEM_HEADER *ih);
void AQFS_ItemHeader_SetOffsetHeaderStart(AQFS_ITEM_HEADER *ih, uint64_t i);

uint64_t AQFS_ItemHeader_GetOffsetDataStart(const AQFS_ITEM_HEADER *ih);
void AQFS_ItemHeader_SetOffsetDataStart(AQFS_ITEM_HEADER *ih, uint64_t i);






#endif
