/****************************************************************************
 * This file is part of the project AqDataBase.
 * AqFinance (c) by 2015 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifndef AQFS_TABLE_H
#define AQFS_TABLE_H


#include <gwenhywfar/list1.h>


#define AQFS_TABLE_HEADER_SIZE      32
#define AQFS_TABLE_MAX_ENTRIES      1024


#define AQFS_TABLE_RUNTIME_FLAGS_DIRTY 0x01




typedef struct AQFS_TABLE AQFS_TABLE;
GWEN_LIST_FUNCTION_DEFS(AQFS_TABLE, AQFS_Table)



#include <aqfilestorage/aqfs_file.h>



AQFS_TABLE *AQFS_Table_new();
void AQFS_Table_free(AQFS_TABLE *ft);

AQFS_TABLE *AQFS_Table_dup(const AQFS_TABLE *ftOrig);


/**
 * Creates a table with the given number of possible entries.
 */
AQFS_TABLE *AQFS_Table_Create(uint64_t maxEntries);


int AQFS_Table_ReadFromBuffer(AQFS_TABLE *ft, const uint8_t *buffer, uint64_t len);
AQFS_TABLE *AQFS_Table_fromBuffer(const uint8_t *buffer, uint64_t len);
uint64_t AQFS_Table_toBuffer(const AQFS_TABLE *ft, uint8_t *buffer, uint64_t len);



uint64_t AQFS_Table_GetMaxEntries(const AQFS_TABLE *ft);
void AQFS_Table_SetMaxEntries(AQFS_TABLE *ft, uint64_t i);

uint64_t AQFS_Table_GetFreeEntries(const AQFS_TABLE *ft);
void AQFS_Table_SetFreeEntries(AQFS_TABLE *ft, uint64_t i);
void AQFS_Table_DecFreeEntries(AQFS_TABLE *ft);
void AQFS_Table_IncFreeEntries(AQFS_TABLE *ft);

uint64_t AQFS_Table_GetHighestEntry(const AQFS_TABLE *ft);
void AQFS_Table_SetHighestEntry(AQFS_TABLE *ft, uint64_t i);
void AQFS_Table_CheckAndSetHighestEntry(AQFS_TABLE *ft, uint64_t i);


uint64_t AQFS_Table_GetOffsetNextTable(const AQFS_TABLE *ft);
void AQFS_Table_SetOffsetNextTable(AQFS_TABLE *ft, uint64_t i);


uint64_t *AQFS_Table_GetPtrEntries(const AQFS_TABLE *ft);

/**
 * Takes over the given pointer.
 */
void AQFS_Table_SetPtrEntries(AQFS_TABLE *ft, uint64_t *ptr);


uint64_t AQFS_Table_GetOffsetTableHeaderStart(const AQFS_TABLE *ft);
void AQFS_Table_SetOffsetTableHeaderStart(AQFS_TABLE *ft, uint64_t i);


uint64_t AQFS_Table_Calculate_Size(const AQFS_TABLE *ft);


uint32_t AQFS_Table_GetRuntimeFlags(const AQFS_TABLE *ft);
void AQFS_Table_SetRuntimeFlags(AQFS_TABLE *ft, uint32_t i);
void AQFS_Table_AddRuntimeFlags(AQFS_TABLE *ft, uint32_t i);
void AQFS_Table_SubRuntimeFlags(AQFS_TABLE *ft, uint32_t i);



AQFS_TABLE_LIST *AQFS_Table_List_dup(const AQFS_TABLE_LIST *ftlOrig);



#endif
