/****************************************************************************
 * This file is part of the project AqDataBase.
 * AqFinance (c) by 2016 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


int AQDB_DbFile_CreateDatabaseStart(AQDB_DB *db) {
  AQDB_DB_FILE *xdb;

  assert(db);
  xdb=GWEN_INHERIT_GETDATA(AQDB_DB, AQDB_DB_FILE, db);
  assert(xdb);

  /* create startTable (table containing database info and list of types) */
  if (1) {
    AQFS_TABLE *ft;
    AQFS_TABLE_LIST *ftl;
    uint64_t offset;
    int rv;
    int64_t rd;

    ft=AQFS_Table_Create(AQDB_DBFILE_TYPETABLE_MAXENTRIES);

    /* append startTable as first table to database */
    rd=AQFS_File_AppendTable(xdb->file, ft);
    if (rd<0) {
      DBG_INFO(AQDB_LOGDOMAIN, "here (%d)", (int) rd);
      AQFS_Table_free(ft);
      return (int) rd;
    }
    AQFS_Table_free(ft);

    /* read first table back */
    ftl=AQFS_Table_List_new();
    offset=AQFS_File_GetFirstItemOffset(xdb->file);
    rv=AQFS_File_ReadTableList(xdb->file, offset, ftl);
    if (rv<0) {
      DBG_INFO(AQDB_LOGDOMAIN, "here (%d)", rv);
      AQFS_Table_List_free(ftl);
      return rv;
    }
    if (xdb->tableListDatabase)
      AQFS_Table_List_free(xdb->tableListDatabase);
    xdb->tableListDatabase=ftl;

    /* add entry for database header to the startTable */
    rd=AQFS_File_AddTableEntry(xdb->file, xdb->tableListDatabase, 0);
    if (rd<0) {
      DBG_INFO(AQDB_LOGDOMAIN, "here (%d)", (int) rd);
      return (int) rd;
    }

    /* add entry for type table to the startTable */
    rd=AQFS_File_AddTableEntry(xdb->file, xdb->tableListDatabase, 0);
    if (rd<0) {
      DBG_INFO(AQDB_LOGDOMAIN, "here (%d)", (int) rd);
      return (int) rd;
    }
  }

  /* create database header */
  if (1) {
    AQDB_DBFILE_HEADER *dbh;
    int rv;

    dbh=AQDB_DbFile_Header_new();
    rv=AQDB_DbFile_WriteHeader(db, dbh);
    if (rv<0) {
      DBG_INFO(AQDB_LOGDOMAIN, "here (%d)", rv);
      return rv;
    }
    if (xdb->header)
      AQDB_DbFile_Header_free(xdb->header);
    xdb->header=dbh;
  }

  /* create type table (table containing list of types) */
  if (1) {
    AQFS_TABLE *ft;
    AQFS_TABLE_LIST *ftl;
    uint64_t offset;
    int rv;
    int64_t rd;

    ft=AQFS_Table_Create(AQDB_DBFILE_TYPETABLE_MAXENTRIES);
    rd=AQFS_File_AppendTable(xdb->file, ft);
    if (rd<0) {
      DBG_INFO(AQDB_LOGDOMAIN, "here (%d)", (int) rd);
      AQFS_Table_free(ft);
      return (int) rd;
    }
    offset=rd;
    AQFS_Table_free(ft);

    /* store offset of type table in startTable */
    AQFS_File_SetTableEntry(xdb->file, xdb->tableListDatabase, AQDB_DBFILE_DBDATA_INDEX_TYPES, offset);
    rv=AQFS_File_WriteDirtyTables(xdb->file, xdb->tableListDatabase);
    if (rv<0) {
      DBG_INFO(AQDB_LOGDOMAIN, "here (%d)", rv);
      return rv;
    }

    /* read type list table back */
    ftl=AQFS_Table_List_new();
    offset=AQFS_File_GetTableEntry(xdb->file, xdb->tableListDatabase, AQDB_DBFILE_DBDATA_INDEX_TYPES);
    rv=AQFS_File_ReadTableList(xdb->file, offset, ftl);
    if (rv<0) {
      DBG_INFO(AQDB_LOGDOMAIN, "here (%d)", rv);
      AQFS_Table_List_free(ftl);
      return rv;
    }
    if (xdb->tableListTypes)
      AQFS_Table_List_free(xdb->tableListTypes);
    xdb->tableListTypes=ftl;

    if (xdb->tables)
      AQDB_DbFile_Table_List_free(xdb->tables);
    xdb->tables=AQDB_DbFile_Table_List_new();
  }

  /* create internal session description type */
  if (1) {
    AQDB_COLUMN_LIST *cl;
    int rv;

    cl=AQDB_SessionDescr_CreateColumnList();
    rv=AQDB_DbFile_ReallyCreateTable(db, AQDB_DBFILE_TYPENAME_SESSIONS, AQDB_ACTION_FLAGS_READ | AQDB_ACTION_FLAGS_WRITE, cl);
    if (rv<0) {
      DBG_INFO(AQDB_LOGDOMAIN, "here (%d)", rv);
      AQDB_Column_List_free(cl);
      return rv;
    }
    AQDB_Column_List_free(cl);
  }

  /* create internal change log type */
  if (1) {
    AQDB_COLUMN_LIST *cl;
    int rv;

    cl=AQDB_DbFile_Changelog_CreateColumnList();
    rv=AQDB_DbFile_ReallyCreateTable(db, AQDB_DBFILE_TYPENAME_CHANGELOG, AQDB_ACTION_FLAGS_READ | AQDB_ACTION_FLAGS_WRITE, cl);
    if (rv<0) {
      DBG_INFO(AQDB_LOGDOMAIN, "here (%d)", rv);
      AQDB_Column_List_free(cl);
      return rv;
    }
    AQDB_Column_List_free(cl);
  }

  /* flush type table lists */
  if (1) {
    int rv;

    rv=AQDB_DbFile_FlushTypeData(db);
    if (rv<0) {
      DBG_INFO(AQDB_LOGDOMAIN, "here (%d)", rv);
      return rv;
    }
  }


  return 0;
}



int AQDB_DbFile_ReadDatabaseStart(AQDB_DB *db) {
  AQDB_DB_FILE *xdb;

  assert(db);
  xdb=GWEN_INHERIT_GETDATA(AQDB_DB, AQDB_DB_FILE, db);
  assert(xdb);

  /* create startTable (table containing database info and list of types) */
  if (1) {
    AQFS_TABLE_LIST *ftl;
    uint64_t offset;
    int rv;

    /* read first table */
    ftl=AQFS_Table_List_new();
    offset=AQFS_File_GetFirstItemOffset(xdb->file);
    rv=AQFS_File_ReadTableList(xdb->file, offset, ftl);
    if (rv<0) {
      DBG_INFO(AQDB_LOGDOMAIN, "here (%d)", rv);
      AQFS_Table_List_free(ftl);
      return rv;
    }
    if (xdb->tableListDatabase)
      AQFS_Table_List_free(xdb->tableListDatabase);
    xdb->tableListDatabase=ftl;
  }

  /* read database header */
  if (1) {
    AQDB_DBFILE_HEADER *dbh;
    int rv;

    dbh=AQDB_DbFile_Header_new();
    rv=AQDB_DbFile_ReadHeader(db, dbh);
    if (rv<0) {
      DBG_INFO(AQDB_LOGDOMAIN, "here (%d)", rv);
      AQDB_DbFile_Header_free(dbh);
      return rv;
    }
    if (xdb->header)
      AQDB_DbFile_Header_free(xdb->header);
    xdb->header=dbh;
  }

  /* read type table (table containing list of types) */
  if (1) {
    AQFS_TABLE_LIST *ftl;
    uint64_t offset;
    int rv;

    /* read type list table back */
    ftl=AQFS_Table_List_new();
    offset=AQFS_File_GetTableEntry(xdb->file, xdb->tableListDatabase, AQDB_DBFILE_DBDATA_INDEX_TYPES);
    rv=AQFS_File_ReadTableList(xdb->file, offset, ftl);
    if (rv<0) {
      DBG_INFO(AQDB_LOGDOMAIN, "here (%d)", rv);
      AQFS_Table_List_free(ftl);
      return rv;
    }
    if (xdb->tableListTypes)
      AQFS_Table_List_free(xdb->tableListTypes);
    xdb->tableListTypes=ftl;

    if (xdb->tables)
      AQDB_DbFile_Table_List_free(xdb->tables);
    xdb->tables=AQDB_DbFile_Table_List_new();
  }

  return 0;
}









int AQDB_DbFile_WriteHeader(AQDB_DB *db, AQDB_DBFILE_HEADER *dbh) {
  AQDB_DB_FILE *xdb;
  uint8_t *ptr;
  uint64_t len;
  uint64_t offset;
  int64_t rd;
  int rv;

  assert(db);
  xdb=GWEN_INHERIT_GETDATA(AQDB_DB, AQDB_DB_FILE, db);
  assert(xdb);

  DBG_INFO(AQDB_LOGDOMAIN, "Writing header");
  ptr=(uint8_t*) malloc(AQDB_DBFILE_HEADER_SIZE);
  assert(ptr);
  rd=AQDB_DbFile_Header_toBuffer(dbh, ptr, AQDB_DBFILE_HEADER_SIZE);
  if (rd<0) {
    DBG_INFO(AQDB_LOGDOMAIN, "here (%d)", (int) rd);
    free(ptr);
    return (int) rd;
  }
  len=rd;

  offset=AQFS_File_GetTableEntry(xdb->file, xdb->tableListDatabase, AQDB_DBFILE_DBDATA_INDEX_HEADER);
  if (offset==0) {
    DBG_INFO(AQDB_LOGDOMAIN, " Creating new header in file");
    /* not yet set, append data */
    rd=AQFS_File_AppendData(xdb->file, AQFS_ITEMHEADER_TYPE_GENERIC_DATA, ptr, len);
    if (rd<0) {
      DBG_INFO(AQDB_LOGDOMAIN, "here (%d)", (int) rd);
      free(ptr);
      return (int) rd;
    }
    offset=rd;

    /* set entry, write table to file */
    AQFS_File_SetTableEntry(xdb->file, xdb->tableListDatabase, AQDB_DBFILE_DBDATA_INDEX_HEADER, offset);
    rv=AQFS_File_WriteDirtyTables(xdb->file, xdb->tableListDatabase);
    if (rv<0) {
      DBG_INFO(AQDB_LOGDOMAIN, "here (%d)", rv);
      return rv;
    }
  }
  else {
    DBG_INFO(AQDB_LOGDOMAIN, " Overwriting existing header in file");
    /* already set, write data */
    rd=AQFS_File_WriteDataAt(xdb->file, offset, AQFS_ITEMHEADER_TYPE_GENERIC_DATA, ptr, len);
    if (rd<0) {
      DBG_INFO(AQDB_LOGDOMAIN, "here (%d)", (int) rd);
      free(ptr);
      return (int) rd;
    }
  }
  free(ptr);

  return 0;
}



int AQDB_DbFile_ReadHeader(AQDB_DB *db, AQDB_DBFILE_HEADER *dbh) {
  AQDB_DB_FILE *xdb;
  uint8_t *ptr=NULL;
  uint64_t offset;
  int64_t rd;
  int rv;

  assert(db);
  xdb=GWEN_INHERIT_GETDATA(AQDB_DB, AQDB_DB_FILE, db);
  assert(xdb);

  DBG_INFO(AQDB_LOGDOMAIN, "Reading header");
  offset=AQFS_File_GetTableEntry(xdb->file, xdb->tableListDatabase, AQDB_DBFILE_DBDATA_INDEX_HEADER);
  if (offset==0) {
    DBG_ERROR(AQDB_LOGDOMAIN, "No header in file");
    return GWEN_ERROR_BAD_DATA;
  }

  rd=AQFS_File_ReadDataAt(xdb->file, offset, &ptr, AQDB_DBFILE_HEADER_SIZE);
  if (rd<0) {
    DBG_INFO(AQFS_LOGDOMAIN, "here (%d)", (int) rd);
    return (int) rd;
  }
  else if (rd==0) {
    DBG_INFO(AQFS_LOGDOMAIN, "Empty data");
    return GWEN_ERROR_BAD_DATA;
  }

  /* now ptr contains a pointer to the header data */
  rv=AQDB_DbFile_Header_ReadFromBuffer(dbh, ptr, rd);
  if (rv<0) {
    DBG_INFO(AQFS_LOGDOMAIN, "here (%d)", rv);
    free(ptr);
    return rv;
  }
  free(ptr);

  /* done */
  return 0;
}



int AQDB_DbFile_ReadTypes(AQDB_DB *db) {
  AQDB_DB_FILE *xdb;
  AQDB_DBFILE_TABLE_LIST *dbtl;
  AQFS_TABLE *ft;
  uint64_t currentTableId=0;

  assert(db);
  xdb=GWEN_INHERIT_GETDATA(AQDB_DB, AQDB_DB_FILE, db);
  assert(xdb);

  DBG_INFO(AQDB_LOGDOMAIN, "Reading types");

  dbtl=AQDB_DbFile_Table_List_new();

  /* read every entry from the type table */
  DBG_INFO(AQDB_LOGDOMAIN, "Table list has %d entries", AQFS_Table_List_GetCount(xdb->tableListTypes));
  ft=AQFS_Table_List_First(xdb->tableListTypes);
  while(ft) {
    uint64_t maxEntries;
    uint64_t idx;
    uint64_t *ptr;

    maxEntries=AQFS_Table_GetMaxEntries(ft);
    DBG_VERBOUS(AQDB_LOGDOMAIN, "Reading entries from table (maxEntries=%d, free entries=%d)",
                (int) maxEntries,
                (int) AQFS_Table_GetFreeEntries(ft));
    ptr=AQFS_Table_GetPtrEntries(ft);
    for (idx=0; idx<maxEntries; idx++) {
      uint64_t offset;

      offset=*(ptr++);
      if (offset>0) {
        AQDB_DBFILE_TABLE *dbt;
        int rv;

        DBG_VERBOUS(AQDB_LOGDOMAIN, "Reading type entry at %llu",
                    (unsigned long long int) offset);
        dbt=AQDB_DbFile_Table_new();
        rv=AQDB_DbFile_ReadTableAt(db, dbt, offset);
        if (rv<0) {
          DBG_INFO(AQDB_LOGDOMAIN, "here (%d)", rv);
          AQDB_DbFile_Table_List_free(dbtl);
          return rv;
        }
        DBG_VERBOUS(AQDB_LOGDOMAIN, "Type at %llu is [%s]",
                    (unsigned long long int) offset,
                 AQDB_DbFile_Table_GetName(dbt));
        AQDB_DbFile_Table_SetTableId(dbt, currentTableId+idx+1);
        AQDB_DbFile_Table_SubOpenFlags(dbt, AQDB_ACTION_FLAGS_MASK);
        DBG_INFO(AQDB_LOGDOMAIN, "Adding type \"%s\" with id %llu",
                 AQDB_DbFile_Table_GetName(dbt),
                 (long long unsigned) (currentTableId+idx+1));
        AQDB_DbFile_Table_List_Add(dbt, dbtl);
      }
    }

    currentTableId+=maxEntries;
    ft=AQFS_Table_List_Next(ft);
  }

  if (xdb->tables)
    AQDB_DbFile_Table_List_free(xdb->tables);
  xdb->tables=dbtl;

  return 0;
}



void AQDB_DbFile_ReleaseInternalStructures(AQDB_DB *db) {
  AQDB_DB_FILE *xdb;

  assert(db);
  xdb=GWEN_INHERIT_GETDATA(AQDB_DB, AQDB_DB_FILE, db);
  assert(xdb);

  DBG_INFO(AQDB_LOGDOMAIN, "Releasing internal structures");

  AQFS_File_free(xdb->file);
  xdb->file=NULL;

  xdb->currentSessionId=0;

  AQDB_SessionDescr_free(xdb->currentSession);
  xdb->currentSession=NULL;

  free(xdb->currentUserName);
  xdb->currentUserName=NULL;

  xdb->openFlags=0;

  AQDB_DbFile_Header_free(xdb->header);
  xdb->header=NULL;

  xdb->tableIdSessionDescr=0;

  AQFS_Table_List_free(xdb->tableListDatabase);
  xdb->tableListDatabase=NULL;

  AQFS_Table_List_free(xdb->tableListTypes);
  xdb->tableListTypes=NULL;

  AQDB_DbFile_Table_List_free(xdb->tables);
  xdb->tables=NULL;
}



int AQDB_DbFile_FlushTypeData(AQDB_DB *db) {
  AQDB_DB_FILE *xdb;
  int rv;

  assert(db);
  xdb=GWEN_INHERIT_GETDATA(AQDB_DB, AQDB_DB_FILE, db);
  assert(xdb);

  DBG_INFO(AQDB_LOGDOMAIN, "Flushing type data");

  /* let all types write their object lists and column lists */
  if (xdb->tables) {
    AQDB_DBFILE_TABLE *dbt;

    dbt=AQDB_DbFile_Table_List_First(xdb->tables);
    while(dbt) {
      AQFS_TABLE_LIST *ftl;

      DBG_INFO(AQDB_LOGDOMAIN, "- Flushing type \"%s\"",
               AQDB_DbFile_Table_GetName(dbt));

      ftl=AQDB_DbFile_Table_GetTableListObjects(dbt);
      if (ftl) {
        if (1) {
          AQFS_TABLE *ft;
          int dirtyTables=0;
          uint64_t numEntries=0;

          ft=AQFS_Table_List_First(ftl);
          while(ft) {
            if (AQFS_Table_GetRuntimeFlags(ft) & AQFS_TABLE_RUNTIME_FLAGS_DIRTY)
              dirtyTables++;
            numEntries+=AQFS_Table_GetMaxEntries(ft)-AQFS_Table_GetFreeEntries(ft);
            ft=AQFS_Table_List_Next(ft);
          }
          DBG_INFO(AQDB_LOGDOMAIN, "  - Writing dirty object tables (%d dirty tables, %llu objects)",
                   dirtyTables, (unsigned long long) numEntries);
        }

        DBG_INFO(AQDB_LOGDOMAIN, "  - Writing dirty object tables");
        rv=AQFS_File_WriteDirtyTables(xdb->file, ftl);
        if (rv<0) {
          DBG_INFO(AQDB_LOGDOMAIN, "here (%d)", rv);
          return rv;
        }
      }
      else {
        DBG_ERROR(AQDB_LOGDOMAIN, "No table list in type \"%s\"",
                  AQDB_DbFile_Table_GetName(dbt));
        return GWEN_ERROR_INTERNAL;
      }

      ftl=AQDB_DbFile_Table_GetTableListColumns(dbt);
      if (ftl) {
        DBG_INFO(AQDB_LOGDOMAIN, "  - Writing dirty column tables");
        rv=AQFS_File_WriteDirtyTables(xdb->file, ftl);
        if (rv<0) {
          DBG_INFO(AQDB_LOGDOMAIN, "here (%d)", rv);
          return rv;
        }
      }

      dbt=AQDB_DbFile_Table_List_Next(dbt);
    }
  }

  /* write type list */
  DBG_INFO(AQDB_LOGDOMAIN, "Writing type list");
  rv=AQFS_File_WriteDirtyTables(xdb->file, xdb->tableListTypes);
  if (rv<0) {
    DBG_INFO(AQDB_LOGDOMAIN, "here (%d)", rv);
    return rv;
  }

  return 0;
}






