/****************************************************************************
 * This file is part of the project AqFinance.
 * AqDataBase (c) by 2015 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifndef AQDB_DBFILE_TABLE_L_H
#define AQDB_DBFILE_TABLE_L_H

#include <aqdatabase/aqdb_db_be.h>
#include <aqdatabase/aqdb_column.h>

#include <aqfilestorage/aqfs_table.h>

#include <gwenhywfar/list1.h>


#define AQDB_DBFILE_TABLE_SIZE 80


typedef struct AQDB_DBFILE_TABLE AQDB_DBFILE_TABLE;
GWEN_LIST_FUNCTION_DEFS(AQDB_DBFILE_TABLE, AQDB_DbFile_Table)



AQDB_DBFILE_TABLE *AQDB_DbFile_Table_new();
void AQDB_DbFile_Table_free(AQDB_DBFILE_TABLE *dbt);


int AQDB_DbFile_Table_ReadFromBuffer(AQDB_DBFILE_TABLE *dbt, const uint8_t *buffer, uint64_t len);
AQDB_DBFILE_TABLE *AQDB_DbFile_Table_fromBuffer(const uint8_t *buffer, uint64_t len);
int64_t AQDB_DbFile_Table_toBuffer(const AQDB_DBFILE_TABLE *dbt, uint8_t *buffer, uint64_t len);


const char *AQDB_DbFile_Table_GetName(const AQDB_DBFILE_TABLE *dbt);
void AQDB_DbFile_Table_SetName(AQDB_DBFILE_TABLE *dbt, const char *s);

uint64_t AQDB_DbFile_Table_GetOffsetColumnTable(const AQDB_DBFILE_TABLE *dbt);
void AQDB_DbFile_Table_SetOffsetColumnTable(AQDB_DBFILE_TABLE *dbt, uint64_t i);

uint64_t AQDB_DbFile_Table_GetOffsetObjectTable(const AQDB_DBFILE_TABLE *dbt);
void AQDB_DbFile_Table_SetOffsetObjectTable(AQDB_DBFILE_TABLE *dbt, uint64_t i);

AQFS_TABLE_LIST *AQDB_DbFile_Table_GetTableListColumns(const AQDB_DBFILE_TABLE *dbt);
void AQDB_DbFile_Table_SetTableListColumns(AQDB_DBFILE_TABLE *dbt, AQFS_TABLE_LIST *ftl);

AQFS_TABLE_LIST *AQDB_DbFile_Table_GetTableListObjects(const AQDB_DBFILE_TABLE *dbt);
void AQDB_DbFile_Table_SetTableListObjects(AQDB_DBFILE_TABLE *dbt, AQFS_TABLE_LIST *ftl);

AQFS_TABLE_LIST *AQDB_DbFile_Table_GetSavedTableListObjects(const AQDB_DBFILE_TABLE *dbt);
void AQDB_DbFile_Table_SetSavedTableListObjects(AQDB_DBFILE_TABLE *dbt, AQFS_TABLE_LIST *ftl);


AQDB_ID AQDB_DbFile_Table_GetTableId(const AQDB_DBFILE_TABLE *dbt);
void AQDB_DbFile_Table_SetTableId(AQDB_DBFILE_TABLE *dbt, AQDB_ID i);

uint32_t AQDB_DbFile_Table_GetOpenFlags(const AQDB_DBFILE_TABLE *dbt);
void AQDB_DbFile_Table_SetOpenFlags(AQDB_DBFILE_TABLE *dbt, uint32_t i);
void AQDB_DbFile_Table_AddOpenFlags(AQDB_DBFILE_TABLE *dbt, uint32_t i);
void AQDB_DbFile_Table_SubOpenFlags(AQDB_DBFILE_TABLE *dbt, uint32_t i);

uint32_t AQDB_DbFile_Table_GetFlags(const AQDB_DBFILE_TABLE *dbt);
void AQDB_DbFile_Table_SetFlags(AQDB_DBFILE_TABLE *dbt, uint32_t i);
void AQDB_DbFile_Table_AddFlags(AQDB_DBFILE_TABLE *dbt, uint32_t i);
void AQDB_DbFile_Table_SubFlags(AQDB_DBFILE_TABLE *dbt, uint32_t i);


AQDB_COLUMN_LIST *AQDB_DbFile_Table_GetColumnList(const AQDB_DBFILE_TABLE *dbt);
void AQDB_DbFile_Table_SetColumnList(AQDB_DBFILE_TABLE *dbt, AQDB_COLUMN_LIST *cl);


int AQDB_DbFile_Table_GetColumnCount(const AQDB_DBFILE_TABLE *dbt);




#endif
